""" A client library for accessing Pigeon """
from .client import AuthenticatedClient
from .client import Client
from .login import Login
from .paginator import Paginator

__all__ = (
    "AuthenticatedClient",
    "Client",
    "Login",
    "Paginator",
)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.group_role import GroupRole


T = TypeVar("T", bound="UserMembership")


@_attrs_define
class UserMembership:
    """UserMembership model

    Attributes:
        id (str):
        role (GroupRole):
    """

    id: str
    role: GroupRole

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        id = self.id
        role = self.role.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UserMembership` from a dict"""
        d = src_dict.copy()
        id = d.pop("id")

        role = GroupRole(d.pop("role"))

        user_membership = cls(
            id=id,
            role=role,
        )

        return user_membership



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collections_items_put_item_body_content_checksum import (
    CollectionsItemsPutItemBodyContentChecksum,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsItemsPutItemBodyContent")


@_attrs_define
class CollectionsItemsPutItemBodyContent:
    """CollectionsItemsPutItemBodyContent model

    Attributes:
        data (str):
        checksum (Union[Unset, CollectionsItemsPutItemBodyContentChecksum]):
        is_base_64_encoded (Union[Unset, bool]):  Default: False.
    """

    data: str
    checksum: Union[Unset, "CollectionsItemsPutItemBodyContentChecksum"] = UNSET
    is_base_64_encoded: Union[Unset, bool] = False

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        data = self.data
        checksum: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.checksum, Unset):
            checksum = self.checksum.to_dict()
        is_base_64_encoded = self.is_base_64_encoded

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
            }
        )
        if checksum is not UNSET:
            field_dict["checksum"] = checksum
        if is_base_64_encoded is not UNSET:
            field_dict["isBase64Encoded"] = is_base_64_encoded

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsPutItemBodyContent` from a dict"""
        d = src_dict.copy()
        data = d.pop("data")

        _checksum = d.pop("checksum", UNSET)
        checksum: Union[Unset, CollectionsItemsPutItemBodyContentChecksum]
        if isinstance(_checksum, Unset):
            checksum = UNSET
        else:
            checksum = CollectionsItemsPutItemBodyContentChecksum.from_dict(_checksum)

        is_base_64_encoded = d.pop("isBase64Encoded", UNSET)

        collections_items_put_item_body_content = cls(
            data=data,
            checksum=checksum,
            is_base_64_encoded=is_base_64_encoded,
        )

        return collections_items_put_item_body_content



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.tasks_schedules_schedules_item import TasksSchedulesSchedulesItem


T = TypeVar("T", bound="TasksSchedules")


@_attrs_define
class TasksSchedules:
    """TasksSchedules model

    Attributes:
        schedules (List['TasksSchedulesSchedulesItem']):
    """

    schedules: List["TasksSchedulesSchedulesItem"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        schedules = []
        for schedules_item_data in self.schedules:
            schedules_item = schedules_item_data.to_dict()
            schedules.append(schedules_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "schedules": schedules,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TasksSchedules` from a dict"""
        d = src_dict.copy()
        schedules = []
        _schedules = d.pop("schedules")
        for schedules_item_data in _schedules:
            schedules_item = TasksSchedulesSchedulesItem.from_dict(schedules_item_data)

            schedules.append(schedules_item)

        tasks_schedules = cls(
            schedules=schedules,
        )

        return tasks_schedules



from .auth import AuthCommands
from .base_commands import BaseCommands
from .cde import CdeCommands
from .config import ConfigCommands
from .pdd import PddCommands
from .tasks import TasksCommands
from .users import UsersCommands


class Commands(BaseCommands):
    auth = AuthCommands
    cdeset = CdeCommands
    config = ConfigCommands
    pdd = PddCommands
    users = UsersCommands
    tasks = TasksCommands



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.data_dictionary_source_item import DataDictionarySourceItem
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataDictionary")


@_attrs_define
class DataDictionary:
    """DataDictionary model

    Attributes:
        source (List[DataDictionarySourceItem]):
        url (str):
        name (Union[Unset, str]):
    """

    source: List[DataDictionarySourceItem]
    url: str
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        source = []
        for source_item_data in self.source:
            source_item = source_item_data.value
            source.append(source_item)

        url = self.url
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "source": source,
                "url": url,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataDictionary` from a dict"""
        d = src_dict.copy()
        source = []
        _source = d.pop("source")
        for source_item_data in _source:
            source_item = DataDictionarySourceItem(source_item_data)

            source.append(source_item)

        url = d.pop("url")

        name = d.pop("name", UNSET)

        data_dictionary = cls(
            source=source,
            url=url,
            name=name,
        )

        return data_dictionary



import pytest
from pypigeon.pigeon_core import AuthenticatedClient


@pytest.fixture
def client():
    return AuthenticatedClient(
        base_url="https://pigeon.test/api/v1", token="test-token"
    )



from pypigeon.pigeon_core.api.admin import admin_grant_admin
from pypigeon.pigeon_core.api.admin import admin_list_admins
from pypigeon.pigeon_core.api.user import user_get_users
from pypigeon.pigeon_core.models import AdminGrantAdminAdminListRequest
from pypigeon.pigeon_core.models import AdminGrants
from pypigeon.pigeon_core.models import UserGetUsersResponse200
from pypigeon.pigeon_core.paginator import Paginator

from .base_commands import BaseCommands


class UsersCommands(BaseCommands):
    """Operations on users"""

    def list(self) -> None:
        """List users."""
        pager = Paginator[UserGetUsersResponse200](user_get_users, self.core)

        data = []
        for page in pager.paginate():
            data.extend([u.to_dict() for u in page.users])

        self._output(data, preferred_type="table")

    def admins_list(self) -> None:
        """Retrieve the current list of admins."""
        admins = admin_list_admins.sync(client=self.core)

        self._output([g.to_dict() for g in admins.grants], preferred_type="table")

    @BaseCommands._with_arg("subject_id")
    @BaseCommands._with_arg("-o", "--operation")
    def admins_add(self) -> None:
        """Add an admin."""
        admins = admin_list_admins.sync(client=self.core)

        if self.args.operation:
            new_operations = {self.args.operation}
        else:
            new_operations = set(admins.operations or [])

        for grant in admins.grants:
            if grant.subject_id == self.args.subject_id:
                grant.operations = list(set(grant.operations) | new_operations)
                break
        else:
            admins.grants.append(
                AdminGrants(
                    subject_id=self.args.subject_id, operations=list(new_operations)
                )
            )

        admin_grant_admin.sync(
            body=AdminGrantAdminAdminListRequest(grants=admins.grants), client=self.core
        )

    @BaseCommands._with_arg("subject_id")
    def admins_revoke(self) -> None:
        """Revoke all admin privileges from a user."""
        admins = admin_list_admins.sync(client=self.core)

        new_grants = [g for g in admins.grants if g.subject_id != self.args.subject_id]
        if new_grants == admins.grants:
            raise ValueError(f"{self.args.subject_id} is not listed in admins")

        admin_grant_admin.sync(
            body=AdminGrantAdminAdminListRequest(grants=new_grants), client=self.core
        )



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.collection_access_level import CollectionAccessLevel
from ..models.collection_metadata import CollectionMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Collection")


@_attrs_define
class Collection:
    """Core collection properties, with IDs

    Attributes:
        access_level (CollectionAccessLevel):
        account_id (str): The account containing this collection
        created_on (datetime.datetime):
        description (str): The collection's full description
        id (str):
        name (str): The collection's short human-readable name
        version_id (str): The collection's version, or LIVE if it is editable
        metadata (Union[Unset, CollectionMetadata]): Arbitrary user-defined metadata, key-value pairs
    """

    access_level: CollectionAccessLevel
    account_id: str
    created_on: datetime.datetime
    description: str
    id: str
    name: str
    version_id: str
    metadata: Union[Unset, "CollectionMetadata"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        access_level = self.access_level.value
        account_id = self.account_id
        created_on = self.created_on.isoformat()
        description = self.description
        id = self.id
        name = self.name
        version_id = self.version_id
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accessLevel": access_level,
                "account_id": account_id,
                "createdOn": created_on,
                "description": description,
                "id": id,
                "name": name,
                "versionId": version_id,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Collection` from a dict"""
        d = src_dict.copy()
        access_level = CollectionAccessLevel(d.pop("accessLevel"))

        account_id = d.pop("account_id")

        created_on = isoparse(d.pop("createdOn"))

        description = d.pop("description")

        id = d.pop("id")

        name = d.pop("name")

        version_id = d.pop("versionId")

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CollectionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CollectionMetadata.from_dict(_metadata)

        collection = cls(
            access_level=access_level,
            account_id=account_id,
            created_on=created_on,
            description=description,
            id=id,
            name=name,
            version_id=version_id,
            metadata=metadata,
        )

        return collection



import ssl
from typing import Any
from typing import Dict
from typing import Optional
from typing import Union

import httpx
from attrs import define
from attrs import evolve
from attrs import field


@define
class Client:
    """A class for keeping track of data related to the API

    The following are accepted as keyword arguments and will be used to construct httpx Clients internally:

        ``base_url``: The base URL for the API, all requests are made to a relative path to this URL

        ``cookies``: A dictionary of cookies to be sent with every request

        ``headers``: A dictionary of headers to be sent with every request

        ``timeout``: The maximum amount of a time a request can take. API functions will raise
        httpx.TimeoutException if this is exceeded.

        ``verify_ssl``: Whether or not to verify the SSL certificate of the API server. This should be True in production,
        but can be set to False for testing purposes.

        ``follow_redirects``: Whether or not to follow redirects. Default value is False.

        ``httpx_args``: A dictionary of additional arguments to be passed to the ``httpx.Client`` and ``httpx.AsyncClient`` constructor.


    Attributes:
        raise_on_unexpected_status: Whether or not to raise an errors.UnexpectedStatus if the API returns a
            status code that was not documented in the source OpenAPI document. Can also be provided as a keyword
            argument to the constructor.
    """

    raise_on_unexpected_status: bool = field(default=False, kw_only=True)
    _base_url: str
    _cookies: Dict[str, str] = field(factory=dict, kw_only=True)
    _headers: Dict[str, str] = field(factory=dict, kw_only=True)
    _timeout: Optional[httpx.Timeout] = field(default=None, kw_only=True)
    _verify_ssl: Union[str, bool, ssl.SSLContext] = field(default=True, kw_only=True)
    _follow_redirects: bool = field(default=False, kw_only=True)
    _httpx_args: Dict[str, Any] = field(factory=dict, kw_only=True)
    _client: Optional[httpx.Client] = field(default=None, init=False)
    _async_client: Optional[httpx.AsyncClient] = field(default=None, init=False)

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        if self._client is not None:
            self._client.headers.update(headers)
        if self._async_client is not None:
            self._async_client.headers.update(headers)
        return evolve(self, headers={**self._headers, **headers})

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        if self._client is not None:
            self._client.cookies.update(cookies)
        if self._async_client is not None:
            self._async_client.cookies.update(cookies)
        return evolve(self, cookies={**self._cookies, **cookies})

    def with_timeout(self, timeout: httpx.Timeout) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        if self._client is not None:
            self._client.timeout = timeout
        if self._async_client is not None:
            self._async_client.timeout = timeout
        return evolve(self, timeout=timeout)

    def set_httpx_client(self, client: httpx.Client) -> "Client":
        """Manually the underlying httpx.Client

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._client = client
        return self

    def get_httpx_client(self) -> httpx.Client:
        """Get the underlying httpx.Client, constructing a new one if not previously set"""
        if self._client is None:
            self._client = httpx.Client(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._client

    def __enter__(self) -> "Client":
        """Enter a context manager for self.client—you cannot enter twice (see httpx docs)"""
        self.get_httpx_client().__enter__()
        return self

    def __exit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for internal httpx.Client (see httpx docs)"""
        self.get_httpx_client().__exit__(*args, **kwargs)

    def set_async_httpx_client(self, async_client: httpx.AsyncClient) -> "Client":
        """Manually the underlying httpx.AsyncClient

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._async_client = async_client
        return self

    def get_async_httpx_client(self) -> httpx.AsyncClient:
        """Get the underlying httpx.AsyncClient, constructing a new one if not previously set"""
        if self._async_client is None:
            self._async_client = httpx.AsyncClient(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._async_client

    async def __aenter__(self) -> "Client":
        """Enter a context manager for underlying httpx.AsyncClient—you cannot enter twice (see httpx docs)"""
        await self.get_async_httpx_client().__aenter__()
        return self

    async def __aexit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for underlying httpx.AsyncClient (see httpx docs)"""
        await self.get_async_httpx_client().__aexit__(*args, **kwargs)


@define
class AuthenticatedClient:
    """A Client which has been authenticated for use on secured endpoints

    The following are accepted as keyword arguments and will be used to construct httpx Clients internally:

        ``base_url``: The base URL for the API, all requests are made to a relative path to this URL

        ``cookies``: A dictionary of cookies to be sent with every request

        ``headers``: A dictionary of headers to be sent with every request

        ``timeout``: The maximum amount of a time a request can take. API functions will raise
        httpx.TimeoutException if this is exceeded.

        ``verify_ssl``: Whether or not to verify the SSL certificate of the API server. This should be True in production,
        but can be set to False for testing purposes.

        ``follow_redirects``: Whether or not to follow redirects. Default value is False.

        ``httpx_args``: A dictionary of additional arguments to be passed to the ``httpx.Client`` and ``httpx.AsyncClient`` constructor.


    Attributes:
        raise_on_unexpected_status: Whether or not to raise an errors.UnexpectedStatus if the API returns a
            status code that was not documented in the source OpenAPI document. Can also be provided as a keyword
            argument to the constructor.
        token: The token to use for authentication
        prefix: The prefix to use for the Authorization header
        auth_header_name: The name of the Authorization header
    """

    raise_on_unexpected_status: bool = field(default=False, kw_only=True)
    _base_url: str
    _cookies: Dict[str, str] = field(factory=dict, kw_only=True)
    _headers: Dict[str, str] = field(factory=dict, kw_only=True)
    _timeout: Optional[httpx.Timeout] = field(default=None, kw_only=True)
    _verify_ssl: Union[str, bool, ssl.SSLContext] = field(default=True, kw_only=True)
    _follow_redirects: bool = field(default=False, kw_only=True)
    _httpx_args: Dict[str, Any] = field(factory=dict, kw_only=True)
    _client: Optional[httpx.Client] = field(default=None, init=False)
    _async_client: Optional[httpx.AsyncClient] = field(default=None, init=False)

    token: str
    prefix: str = "Bearer"
    auth_header_name: str = "Authorization"

    def with_headers(self, headers: Dict[str, str]) -> "AuthenticatedClient":
        """Get a new client matching this one with additional headers"""
        if self._client is not None:
            self._client.headers.update(headers)
        if self._async_client is not None:
            self._async_client.headers.update(headers)
        return evolve(self, headers={**self._headers, **headers})

    def with_cookies(self, cookies: Dict[str, str]) -> "AuthenticatedClient":
        """Get a new client matching this one with additional cookies"""
        if self._client is not None:
            self._client.cookies.update(cookies)
        if self._async_client is not None:
            self._async_client.cookies.update(cookies)
        return evolve(self, cookies={**self._cookies, **cookies})

    def with_timeout(self, timeout: httpx.Timeout) -> "AuthenticatedClient":
        """Get a new client matching this one with a new timeout (in seconds)"""
        if self._client is not None:
            self._client.timeout = timeout
        if self._async_client is not None:
            self._async_client.timeout = timeout
        return evolve(self, timeout=timeout)

    def set_httpx_client(self, client: httpx.Client) -> "AuthenticatedClient":
        """Manually the underlying httpx.Client

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._client = client
        return self

    def get_httpx_client(self) -> httpx.Client:
        """Get the underlying httpx.Client, constructing a new one if not previously set"""
        if self._client is None:
            self._headers[self.auth_header_name] = (
                f"{self.prefix} {self.token}" if self.prefix else self.token
            )
            self._client = httpx.Client(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._client

    def __enter__(self) -> "AuthenticatedClient":
        """Enter a context manager for self.client—you cannot enter twice (see httpx docs)"""
        self.get_httpx_client().__enter__()
        return self

    def __exit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for internal httpx.Client (see httpx docs)"""
        self.get_httpx_client().__exit__(*args, **kwargs)

    def set_async_httpx_client(
        self, async_client: httpx.AsyncClient
    ) -> "AuthenticatedClient":
        """Manually the underlying httpx.AsyncClient

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._async_client = async_client
        return self

    def get_async_httpx_client(self) -> httpx.AsyncClient:
        """Get the underlying httpx.AsyncClient, constructing a new one if not previously set"""
        if self._async_client is None:
            self._headers[self.auth_header_name] = (
                f"{self.prefix} {self.token}" if self.prefix else self.token
            )
            self._async_client = httpx.AsyncClient(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._async_client

    async def __aenter__(self) -> "AuthenticatedClient":
        """Enter a context manager for underlying httpx.AsyncClient—you cannot enter twice (see httpx docs)"""
        await self.get_async_httpx_client().__aenter__()
        return self

    async def __aexit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for underlying httpx.AsyncClient (see httpx docs)"""
        await self.get_async_httpx_client().__aexit__(*args, **kwargs)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.item import Item
from ..models.pagination import Pagination


T = TypeVar("T", bound="CollectionsItemsListItemsResponse200")


@_attrs_define
class CollectionsItemsListItemsResponse200:
    """CollectionsItemsListItemsResponse200 model

    Attributes:
        items (List['Item']):
        pagination (Pagination):
    """

    items: List["Item"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "items": items,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsListItemsResponse200` from a dict"""
        d = src_dict.copy()
        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = Item.from_dict(items_item_data)

            items.append(items_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        collections_items_list_items_response_200 = cls(
            items=items,
            pagination=pagination,
        )

        return collections_items_list_items_response_200



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_concept_applies_to import DataElementConceptAppliesTo


T = TypeVar("T", bound="DataElementConcept")


@_attrs_define
class DataElementConcept:
    """Reference to an entry in a concept repository.

    Attributes:
        applies_to (DataElementConceptAppliesTo):
        name (str): The human-readable definition of the concept.
        origin (str): The repository where this concept can be found, either as a commonly known name, or as a URL.
        origin_id (str): The ID of the concept in the concept repository.
    """

    applies_to: DataElementConceptAppliesTo
    name: str
    origin: str
    origin_id: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        applies_to = self.applies_to.value
        name = self.name
        origin = self.origin
        origin_id = self.origin_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "appliesTo": applies_to,
                "name": name,
                "origin": origin,
                "originId": origin_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementConcept` from a dict"""
        d = src_dict.copy()
        applies_to = DataElementConceptAppliesTo(d.pop("appliesTo"))

        name = d.pop("name")

        origin = d.pop("origin")

        origin_id = d.pop("originId")

        data_element_concept = cls(
            applies_to=applies_to,
            name=name,
            origin=origin,
            origin_id=origin_id,
        )

        return data_element_concept



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="QueryDataElementRequest")


@_attrs_define
class QueryDataElementRequest:
    """QueryDataElementRequest model

    Attributes:
        query (str):
        parser_name (Union[Unset, str]):
    """

    query: str
    parser_name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        query = self.query
        parser_name = self.parser_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "query": query,
            }
        )
        if parser_name is not UNSET:
            field_dict["parserName"] = parser_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`QueryDataElementRequest` from a dict"""
        d = src_dict.copy()
        query = d.pop("query")

        parser_name = d.pop("parserName", UNSET)

        query_data_element_request = cls(
            query=query,
            parser_name=parser_name,
        )

        return query_data_element_request



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="SessionToken")


@_attrs_define
class SessionToken:
    """SessionToken model

    Attributes:
        activation_code (str):
        session_id (str):
    """

    activation_code: str
    session_id: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        activation_code = self.activation_code
        session_id = self.session_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "activation_code": activation_code,
                "session_id": session_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SessionToken` from a dict"""
        d = src_dict.copy()
        activation_code = d.pop("activation_code")

        session_id = d.pop("session_id")

        session_token = cls(
            activation_code=activation_code,
            session_id=session_id,
        )

        return session_token



from enum import Enum


class TaskScheduleDefFrequencyType0(str, Enum):
    DAILY = "daily"
    HOURLY = "hourly"
    MONTHLY = "monthly"
    WEEKLY = "weekly"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class FederationCollectionPageType(str, Enum):
    ORDEREDCOLLECTION = "OrderedCollection"
    ORDEREDCOLLECTIONPAGE = "OrderedCollectionPage"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.data_element_definition_definition_type import (
    DataElementDefinitionDefinitionType,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataElementDefinition")


@_attrs_define
class DataElementDefinition:
    """A single definition applicable to this data element.

    Attributes:
        definition_type (DataElementDefinitionDefinitionType):
        text (str):
        other_type (Union[Unset, str]):
        source (Union[Unset, str]):
    """

    definition_type: DataElementDefinitionDefinitionType
    text: str
    other_type: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        definition_type = self.definition_type.value
        text = self.text
        other_type = self.other_type
        source = self.source

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "definitionType": definition_type,
                "text": text,
            }
        )
        if other_type is not UNSET:
            field_dict["otherType"] = other_type
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementDefinition` from a dict"""
        d = src_dict.copy()
        definition_type = DataElementDefinitionDefinitionType(d.pop("definitionType"))

        text = d.pop("text")

        other_type = d.pop("otherType", UNSET)

        source = d.pop("source", UNSET)

        data_element_definition = cls(
            definition_type=definition_type,
            text=text,
            other_type=other_type,
            source=source,
        )

        return data_element_definition



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.pagination import Pagination
from ..models.user import User


T = TypeVar("T", bound="UserGetUsersResponse200")


@_attrs_define
class UserGetUsersResponse200:
    """UserGetUsersResponse200 model

    Attributes:
        pagination (Pagination):
        users (List['User']):
    """

    pagination: "Pagination"
    users: List["User"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        pagination = self.pagination.to_dict()
        users = []
        for users_item_data in self.users:
            users_item = users_item_data.to_dict()
            users.append(users_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "pagination": pagination,
                "users": users,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UserGetUsersResponse200` from a dict"""
        d = src_dict.copy()
        pagination = Pagination.from_dict(d.pop("pagination"))

        users = []
        _users = d.pop("users")
        for users_item_data in _users:
            users_item = User.from_dict(users_item_data)

            users.append(users_item)

        user_get_users_response_200 = cls(
            pagination=pagination,
            users=users,
        )

        return user_get_users_response_200



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="NewUser")


@_attrs_define
class NewUser:
    """NewUser model

    Attributes:
        given_name (str):
        last_name (str):
        password (str):
        user_email (str):
        user_name (str):
    """

    given_name: str
    last_name: str
    password: str
    user_email: str
    user_name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        given_name = self.given_name
        last_name = self.last_name
        password = self.password
        user_email = self.user_email
        user_name = self.user_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "given_name": given_name,
                "last_name": last_name,
                "password": password,
                "user_email": user_email,
                "user_name": user_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`NewUser` from a dict"""
        d = src_dict.copy()
        given_name = d.pop("given_name")

        last_name = d.pop("last_name")

        password = d.pop("password")

        user_email = d.pop("user_email")

        user_name = d.pop("user_name")

        new_user = cls(
            given_name=given_name,
            last_name=last_name,
            password=password,
            user_email=user_email,
            user_name=user_name,
        )

        return new_user



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.new_collection_metadata import NewCollectionMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="NewCollection")


@_attrs_define
class NewCollection:
    """New collection request

    Attributes:
        description (str): The collection's full description
        name (str): The collection's short human-readable name
        account_id (Union[Unset, str]): The account into which the collection should be created
        metadata (Union[Unset, NewCollectionMetadata]): Arbitrary user-defined metadata, key-value pairs
    """

    description: str
    name: str
    account_id: Union[Unset, str] = UNSET
    metadata: Union[Unset, "NewCollectionMetadata"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        description = self.description
        name = self.name
        account_id = self.account_id
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
                "name": name,
            }
        )
        if account_id is not UNSET:
            field_dict["account_id"] = account_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`NewCollection` from a dict"""
        d = src_dict.copy()
        description = d.pop("description")

        name = d.pop("name")

        account_id = d.pop("account_id", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, NewCollectionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = NewCollectionMetadata.from_dict(_metadata)

        new_collection = cls(
            description=description,
            name=name,
            account_id=account_id,
            metadata=metadata,
        )

        return new_collection



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.system_configuration_authentication_additional_property_type_0 import (
    SystemConfigurationAuthenticationAdditionalPropertyType0,
)
from ..models.system_configuration_authentication_additional_property_type_1 import (
    SystemConfigurationAuthenticationAdditionalPropertyType1,
)
from ..models.system_configuration_authentication_additional_property_type_2 import (
    SystemConfigurationAuthenticationAdditionalPropertyType2,
)


T = TypeVar("T", bound="SystemConfigurationAuthentication")


@_attrs_define
class SystemConfigurationAuthentication:
    """SystemConfigurationAuthentication model"""

    additional_properties: Dict[
        str,
        Union[
            "SystemConfigurationAuthenticationAdditionalPropertyType0",
            "SystemConfigurationAuthenticationAdditionalPropertyType1",
            "SystemConfigurationAuthenticationAdditionalPropertyType2",
        ],
    ] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            if isinstance(
                prop, SystemConfigurationAuthenticationAdditionalPropertyType0
            ):
                field_dict[prop_name] = prop.to_dict()
            elif isinstance(
                prop, SystemConfigurationAuthenticationAdditionalPropertyType1
            ):
                field_dict[prop_name] = prop.to_dict()
            else:
                field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationAuthentication` from a dict"""
        d = src_dict.copy()
        system_configuration_authentication = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(
                data: object,
            ) -> Union[
                "SystemConfigurationAuthenticationAdditionalPropertyType0",
                "SystemConfigurationAuthenticationAdditionalPropertyType1",
                "SystemConfigurationAuthenticationAdditionalPropertyType2",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    additional_property_type_0 = SystemConfigurationAuthenticationAdditionalPropertyType0.from_dict(
                        data
                    )

                    return additional_property_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    additional_property_type_1 = SystemConfigurationAuthenticationAdditionalPropertyType1.from_dict(
                        data
                    )

                    return additional_property_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                additional_property_type_2 = (
                    SystemConfigurationAuthenticationAdditionalPropertyType2.from_dict(
                        data
                    )
                )

                return additional_property_type_2

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        system_configuration_authentication.additional_properties = (
            additional_properties
        )
        return system_configuration_authentication

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(
        self, key: str
    ) -> Union[
        "SystemConfigurationAuthenticationAdditionalPropertyType0",
        "SystemConfigurationAuthenticationAdditionalPropertyType1",
        "SystemConfigurationAuthenticationAdditionalPropertyType2",
    ]:
        return self.additional_properties[key]

    def __setitem__(
        self,
        key: str,
        value: Union[
            "SystemConfigurationAuthenticationAdditionalPropertyType0",
            "SystemConfigurationAuthenticationAdditionalPropertyType1",
            "SystemConfigurationAuthenticationAdditionalPropertyType2",
        ],
    ) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.system_configuration_datastores_additional_property_type_1_type import (
    SystemConfigurationDatastoresAdditionalPropertyType1Type,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SystemConfigurationDatastoresAdditionalPropertyType1")


@_attrs_define
class SystemConfigurationDatastoresAdditionalPropertyType1:
    """SystemConfigurationDatastoresAdditionalPropertyType1 model

    Attributes:
        bucket (str):
        access_key (Union[Unset, str]):
        cache_max (Union[Unset, int]):  Default: 1073741824.
        enabled (Union[Unset, bool]):  Default: True.
        endpoint_url (Union[Unset, str]):
        is_default (Union[Unset, bool]):  Default: False.
        secret_key (Union[Unset, str]):
        type (Union[Unset, SystemConfigurationDatastoresAdditionalPropertyType1Type]):
    """

    bucket: str
    access_key: Union[Unset, str] = UNSET
    cache_max: Union[Unset, int] = 1073741824
    enabled: Union[Unset, bool] = True
    endpoint_url: Union[Unset, str] = UNSET
    is_default: Union[Unset, bool] = False
    secret_key: Union[Unset, str] = UNSET
    type: Union[Unset, SystemConfigurationDatastoresAdditionalPropertyType1Type] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        bucket = self.bucket
        access_key = self.access_key
        cache_max = self.cache_max
        enabled = self.enabled
        endpoint_url = self.endpoint_url
        is_default = self.is_default
        secret_key = self.secret_key
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bucket": bucket,
            }
        )
        if access_key is not UNSET:
            field_dict["access_key"] = access_key
        if cache_max is not UNSET:
            field_dict["cache_max"] = cache_max
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if endpoint_url is not UNSET:
            field_dict["endpoint_url"] = endpoint_url
        if is_default is not UNSET:
            field_dict["is_default"] = is_default
        if secret_key is not UNSET:
            field_dict["secret_key"] = secret_key
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationDatastoresAdditionalPropertyType1` from a dict"""
        d = src_dict.copy()
        bucket = d.pop("bucket")

        access_key = d.pop("access_key", UNSET)

        cache_max = d.pop("cache_max", UNSET)

        enabled = d.pop("enabled", UNSET)

        endpoint_url = d.pop("endpoint_url", UNSET)

        is_default = d.pop("is_default", UNSET)

        secret_key = d.pop("secret_key", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, SystemConfigurationDatastoresAdditionalPropertyType1Type]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = SystemConfigurationDatastoresAdditionalPropertyType1Type(_type)

        system_configuration_datastores_additional_property_type_1 = cls(
            bucket=bucket,
            access_key=access_key,
            cache_max=cache_max,
            enabled=enabled,
            endpoint_url=endpoint_url,
            is_default=is_default,
            secret_key=secret_key,
            type=type,
        )

        system_configuration_datastores_additional_property_type_1.additional_properties = (
            d
        )
        return system_configuration_datastores_additional_property_type_1

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.query_data_elements_element_map import QueryDataElementsElementMap
from ..models.query_data_elements_error_map import QueryDataElementsErrorMap
from ..models.query_data_elements_frames_item import QueryDataElementsFramesItem


T = TypeVar("T", bound="QueryDataElements")


@_attrs_define
class QueryDataElements:
    """QueryDataElements model

    Attributes:
        element_map (QueryDataElementsElementMap):
        error_map (QueryDataElementsErrorMap):
        frames (List['QueryDataElementsFramesItem']):
    """

    element_map: "QueryDataElementsElementMap"
    error_map: "QueryDataElementsErrorMap"
    frames: List["QueryDataElementsFramesItem"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        element_map = self.element_map.to_dict()
        error_map = self.error_map.to_dict()
        frames = []
        for frames_item_data in self.frames:
            frames_item = frames_item_data.to_dict()
            frames.append(frames_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "elementMap": element_map,
                "errorMap": error_map,
                "frames": frames,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`QueryDataElements` from a dict"""
        d = src_dict.copy()
        element_map = QueryDataElementsElementMap.from_dict(d.pop("elementMap"))

        error_map = QueryDataElementsErrorMap.from_dict(d.pop("errorMap"))

        frames = []
        _frames = d.pop("frames")
        for frames_item_data in _frames:
            frames_item = QueryDataElementsFramesItem.from_dict(frames_item_data)

            frames.append(frames_item)

        query_data_elements = cls(
            element_map=element_map,
            error_map=error_map,
            frames=frames,
        )

        return query_data_elements



from enum import Enum


class TaskResultStatus(str, Enum):
    CANCELED = "CANCELED"
    CANCELED_FAILED = "CANCELED_FAILED"
    CANCELED_PAST_DEADLINE = "CANCELED_PAST_DEADLINE"
    FAILED = "FAILED"
    FAILED_RETRY = "FAILED_RETRY"
    OK = "OK"
    TIMEOUT_CANCELED = "TIMEOUT_CANCELED"
    TIMEOUT_FAILED = "TIMEOUT_FAILED"
    TIMEOUT_RETRY = "TIMEOUT_RETRY"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.pagination import Pagination
from ..models.search_dictionaries_by_item_column_hits import (
    SearchDictionariesByItemColumnHits,
)


T = TypeVar("T", bound="SearchDictionariesByItemResponse")


@_attrs_define
class SearchDictionariesByItemResponse:
    """SearchDictionariesByItemResponse model

    Attributes:
        columns (List['SearchDictionariesByItemColumnHits']):
        pagination (Pagination):
    """

    columns: List["SearchDictionariesByItemColumnHits"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        columns = []
        for columns_item_data in self.columns:
            columns_item = columns_item_data.to_dict()
            columns.append(columns_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "columns": columns,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchDictionariesByItemResponse` from a dict"""
        d = src_dict.copy()
        columns = []
        _columns = d.pop("columns")
        for columns_item_data in _columns:
            columns_item = SearchDictionariesByItemColumnHits.from_dict(
                columns_item_data
            )

            columns.append(columns_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        search_dictionaries_by_item_response = cls(
            columns=columns,
            pagination=pagination,
        )

        return search_dictionaries_by_item_response



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.error import Error
from ..models.pagination import Pagination
from ..models.table import Table


T = TypeVar("T", bound="CollectionsTablesListTablesResponse200")


@_attrs_define
class CollectionsTablesListTablesResponse200:
    """CollectionsTablesListTablesResponse200 model

    Attributes:
        errors (List['Error']):
        pagination (Pagination):
        tables (List['Table']):
    """

    errors: List["Error"]
    pagination: "Pagination"
    tables: List["Table"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        errors = []
        for errors_item_data in self.errors:
            errors_item = errors_item_data.to_dict()
            errors.append(errors_item)

        pagination = self.pagination.to_dict()
        tables = []
        for tables_item_data in self.tables:
            tables_item = tables_item_data.to_dict()
            tables.append(tables_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "errors": errors,
                "pagination": pagination,
                "tables": tables,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsTablesListTablesResponse200` from a dict"""
        d = src_dict.copy()
        errors = []
        _errors = d.pop("errors")
        for errors_item_data in _errors:
            errors_item = Error.from_dict(errors_item_data)

            errors.append(errors_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        tables = []
        _tables = d.pop("tables")
        for tables_item_data in _tables:
            tables_item = Table.from_dict(tables_item_data)

            tables.append(tables_item)

        collections_tables_list_tables_response_200 = cls(
            errors=errors,
            pagination=pagination,
            tables=tables,
        )

        return collections_tables_list_tables_response_200



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Bundle")


@_attrs_define
class Bundle:
    """Bundle model

    Attributes:
        elements (List[str]):
        id (str):
        name (str):
        version (Union[Unset, str]):
    """

    elements: List[str]
    id: str
    name: str
    version: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        elements = self.elements

        id = self.id
        name = self.name
        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "elements": elements,
                "id": id,
                "name": name,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Bundle` from a dict"""
        d = src_dict.copy()
        elements = cast(List[str], d.pop("elements"))

        id = d.pop("id")

        name = d.pop("name")

        version = d.pop("version", UNSET)

        bundle = cls(
            elements=elements,
            id=id,
            name=name,
            version=version,
        )

        return bundle
